/**
 * @file main.cpp
 * @brief Simple example of universal kriging estimator
 * @author Tomohiko Mukai
 */

#include <iostream>
#include <fstream>

#include "geostat.h"

int main(void)
{
	// dimension of control space
	const size_t dim_cspace = 2;
	// number of samples
	const size_t num_samples = 16;

	// kriging predictor
	CGeostat krig;

	// initialize kriging predictor
	krig.initialize(num_samples, dim_cspace);


	//
	// define a domain of control space
	//
	gsl_vector *lower = gsl_vector_alloc(dim_cspace);
	gsl_vector *upper = gsl_vector_alloc(dim_cspace);
	//
	//control parameter c1 : 0 <= c1 <= 4.0
	gsl_vector_set(lower, 0, 0.0);
	gsl_vector_set(upper, 0, 4.0);
	//
	// control parameter c2 : 0 <= c2 <= 5.0
	gsl_vector_set(lower, 1, 0.0);
	gsl_vector_set(upper, 1, 5.0);
	//
	// set domain
	krig.setDomain(lower, upper);
	//
	gsl_vector_free(lower);
	gsl_vector_free(upper);


	//
	// sample data
	//
	gsl_vector *c = gsl_vector_alloc(dim_cspace);

	double tmp, pred, var;
	for (size_t i = 0;i < num_samples;i++)
	{
		//
		// regular sampling point...
		gsl_vector_set(c, 0, static_cast<double>(i / 4));
		gsl_vector_set(c, 1, static_cast<double>(i % 4));
		//
		// coordinate of i-th sample
		krig.setCoordinate(i, c);

		//
		// randomly generated sample data
		tmp = static_cast<double>(rand()) / static_cast<double>(RAND_MAX);
		//
		// data of i-th sample
		krig.setData(i, tmp);
	}


	//
	// precomputation of statistical analysis
	//
	//  using Spherical variogram model
	//  power coefficient = 0 (only use for stable variogram model)
	//  step width (h_step = 0.1)
	krig.estimate(CVariogram::VARIO_SPH, 0, 0.1);


	//
	// output file stream
	std::ofstream ofkrig;
	ofkrig.open("krig.csv");

	//
	// prediction: 100x100 sample point
	//
	for (size_t i = 0;i < 100;i++)
	{
		// target parameter c1
		gsl_vector_set(c, 0, 0.04 * i);
		for (size_t j = 0;j < 100;j++)
		{
			// target parameter c2
			gsl_vector_set(c, 1, 0.04 * j);

			// predictive value and estimation variance at parameter c
			krig.getPredictData(pred, var, c);
			ofkrig << pred << ", ";
		}
		ofkrig << std::endl;
	}
	ofkrig.close();
	
	gsl_vector_free(c);

	return 0;
}
