/**
 * @file variogram.h
 * @brief declaration: variogram class
 * @author Tomohiko Mukai
 */
#ifndef	VARIOGRAM_HEADER_FILE_
#define	VARIOGRAM_HEADER_FILE_

#include <vector>

/**
 * @class CVariogram Variogram.h
 * @brief experimental/theoretical variogram class
 * @author Tomohiko Mukai
 */
class CVariogram
{
//
// member enumeration
public:
	/**
	 *	@enum VariogramModel
	 */
	enum VariogramModel
	{
		//! not estimated
		VARIO_NONE,
		//! spherical model
		VARIO_SPH,
		//! stable model
		VARIO_STB,
		//! number of models
		VARIO_NUM
	};

//
// member classes
public:
	/**
	 * @struct VarioItm
	 * @brief structure of distance and dissimilarity
	 */
	struct VarioItm
	{
		//! distance
		double distance;
		//! dissimilarity
		double dissimilarity;
	};

//
// variables
private:
	//! theoretical variogram model
	int m_model;
	//! number of samples
	size_t m_samples;
	//! container for distance
	double *m_pDistance;
	//! container for experimental variogram
	double *m_pVariogram;

	//
	// papameters of theoretical variogram
	//
	//! nugget
	double m_nugget;
	//! sill
	double m_sill;
	//! range
	double m_range;
	//! power coefficient of stable model
	double m_power;

//
// constructor / destructor
//
public:
	// default constructor
	CVariogram(void);
	// destructor
	virtual ~CVariogram(void);

//
// attributes
public:
	/**
	 *	@fn bool isActive(void) const
	 *	@retval true active
	 *	@retval false inactive
	 */
	bool isActive(void) const
		{
			return m_pDistance != NULL && m_pVariogram;
		}
	/**
	 *	@fn bool isEstimated(void) const
	 *	@retval true estimated
	 *	@retval false not estimated
	 */
	bool isEstimated(void) const
		{
			return m_model != VARIO_NONE;
		}
	/**
	 *	@fn size_t samples(void) const
	 *	@return number of samples
	 */
	size_t samples(void) const
		{
			return m_samples;
		}
	/**
	 *	@fn double maxDistance(void) const
	 *	@return maximum distance
	 */
	double maxDistance(void) const
		{
			return m_pDistance[m_samples - 1];
		}
	/**
	 *	@fn double minDistance(void) const
	 *	@return minimum distance
	 */
	double minDistance(void) const
		{
			return m_pDistance[0];
		}
	/**
	 *	@fn double nugget(void) const
	 *	@return nugget
	 */
	double nugget(void) const
		{
			return m_nugget;
		}
	/**
	 *	@fn double sill(void) const
	 *	@return sill
	 */
	double sill(void) const
		{
			return m_sill;
		}
	/**
	 *	@fn double range(void) const
	 *	@return range
	 */
	double range(void) const
		{
			return m_range;
		}
	/**
	 *	@fn double power(void) const
	 *	@return power coefficient (only for stable models)
	 */
	double power(void) const
		{
			return m_power;
		}
	/**
	 *	@fn int modelType(void) const
	 *	@return type of theoretical variogram
	 */
	int modelType(void) const
		{
			return m_model;
		}
	// get number of samples whose distance is less than given value
	size_t countLessDist(double cap) const;

//
// data reader
public:
	// get sample data (distance & dissimilarity)
	bool getSample(size_t smpl, double &dist, double &vario) const;
	// get theoretical dissimilarity
	double getModelData(double dist) const;
	// get covariance corresponding to theoretical dissimilarity
	double getModelCovariance(double dist) const;

//
// data writer
public:
	// set sample data (distance & dissimilarity)
	bool setSample(const std::vector<CVariogram::VarioItm> &vecSample);
	// set parameters of theoretical variogram (nugget, sill, range, power, and so on)
	bool setModel(int model, double nugget, double sill, double range, double power, double step, double maxdist);

//
// manipulator
protected:
	// sort container for dissimilarity by distance
	bool sortByDistance(void);
public:
	// estimate theoretical variogram by non-linear least square fitting
	int estimateModel(int model, double &nugget, double &sill, double &range, double power, double maxdist =1.0e6);

//
// memory manager
private:
	// memory allocation
	bool allocate(size_t smpl);
	// memory destruction
	void destroy(void);
};

#endif	//VARIOGRAM_HEADER_FILE_
